import { joinRTCRoom, leaveRTCRoom } from '../services/index'
import { disconnect } from '@rongcloud/imlib-next'
import { RCRTCCode, RCRTCRoom, RCAudioOutputDevice } from '@rongcloud/plugin-wechat-rtc'

wx.setKeepScreenOn({ keepScreenOn: true })

/**
 * @type {RCRTCRoom}
 */
let room

const printRoomData = () => {
    console.log('摄像头是否打开: ', room.isCameraOpen());
    console.log('麦克风是否打开: ', room.isMicphoneOpen());
    console.log('房间是否被销毁: ', room.isDestroyed());
    console.log('房间号: ', room.getRoomId());
    console.log('当前用户: ', room.getCrtUserId());
    console.log('房间内远端用户: ', room.getRemoteUserIds());
    console.log('房间内远端资源: ', room.getRemoteStreams());
}

Page({
  data: {
    roomId: '',
    remoteStreams: [],
    isJoined: false
  },
  inputRoomId(e) {
    this.setData({ roomId: e.detail.value })
  },
  onLoad (options) {
      const { userId } = options
      wx.setNavigationBarTitle({ title: `当前用户: ${userId}` })
  },
  async onUnload() {
    await this.leaveRoom();
    await disconnect()
  },
  async joinRoom() {
      wx.showLoading({ title: '加入房间...' })
      const { roomId } = this.data
      const { code, data } = await joinRTCRoom(roomId)
      wx.hideLoading()

      if (code !== RCRTCCode.SUCCESS) {
          wx.showToast({ title: `加入房间失败: ${code}`, icon: 'none' })
          return
      }

      room = data.room
      const ctx = this

      // 添加房间事件监听  
      room.registerRoomEventListener({
          onKickOff(byServer, stat) {
              wx.showToast({ title: '您已被踢出房间', icon: 'none' })
              ctx._leaveHandle()
          },
          onMessageReceive(name, content, senderId, messageUId) {
              console.info(`接收到消息 -> name: ${name}, content: ${content}`)
          },
          onUserJoin(userIds) {
            userIds.forEach(id => {
                wx.showToast({ title: `用户 ${id} 加入房间`, icon: 'none' })
            })
          },
          onUserLeave(userIds) {
            let { remoteStreams } = ctx.data
            userIds.forEach(id => {
                wx.showToast({ title: `用户 ${id} 离开房间`, icon: 'none' })
                remoteStreams = remoteStreams.filter(item => !item.msid.startsWith(id))
            })
            ctx.setData({ remoteStreams })
          },
          onAudioMuteChange(stream) {
            const userId = stream.getUserId()
            const muted = stream.isOwnerMuteAudio()
            wx.showToast({ title: `用户 ${userId} ${ muted ? '禁用' : '启用' } 了音频`, icon: 'none' })
          },
          onVideoMuteChange(stream) {
            const userId = stream.getUserId()
            const disabled = stream.isOwnerDisableVideo()
            wx.showToast({ title: `用户 ${userId} ${ disabled ? '禁用' : '启用' } 了视频`, icon: 'none' })
          },
          onStreamPublish() {
            const streams = room.getRemoteStreams()
            ctx.setData({
                remoteStreams: streams.map((stream) => ({ msid: stream.getMsid(), stream }))
            })
          },
          onStreamUnpublish() {
            const streams = room.getRemoteStreams()
            ctx.setData({
                remoteStreams: streams.map((stream) => ({ msid: stream.getMsid(), stream }))
            })
          }
      })

      // 注册网络动态、质量数据监听
      room.registerReportListener((stat) => {
          console.log(`report ->`, stat)
      })
      const streams = room.getRemoteStreams()

      this.setData({
          isJoined: true,
          remoteStreams: streams.map((stream) => ({ msid: stream.getMsid(), stream }))
    })
  },
  _leaveHandle() {
    this.setData({
        remoteStreams: [],
        isJoined: false,
        roomId: ''
      });
    room = undefined
  },
  async leaveRoom () {
      if (!room) {
        return
      }
      wx.showLoading({ title: '正在退出房间...' })
      await leaveRTCRoom(room)
      this._leaveHandle()
      wx.showToast({ title: 'Leave Successfully', icon: 'none' });
  },
  async publish () {
    const { code } =  await room.publishStream()
    if (code !== RCRTCCode.SUCCESS) {
      wx.showToast({ title: `publish failed ${code}`, icon: 'none' })
    } else {
      wx.showToast({ title: 'Success' })
    }
  },
  async unpublish () {
    const { code } = await room.unpublishStream()
    if (code === RCRTCCode.SUCCESS) {
        wx.showToast({ title: 'unpublish success', icon: 'none' })
    } else {
        wx.showToast({ title: `unpublish failed: ${code}` })
    }
  },
  getStreamByMsId (e) {
    const msId = e.target.dataset.id;
    const { remoteStreams } = this.data;
    const item = remoteStreams.find(item => item.msid === msId)
    return item.stream
  },
  async subscribe (e, subTiny) {
    wx.showLoading({ title: '订阅中...', mask: true });
    const stream = this.getStreamByMsId(e);

    const { code } = await room.subscribe([{
        stream,
        subTiny
    }])
    wx.hideLoading()

    if (code !== RCRTCCode.SUCCESS) {
        wx.showToast({ title: `subscribe failed: ${code}`, icon: 'none' })
    } else {
        wx.showToast({ title: 'subscribe success', icon: 'none' })
    }
  },
  async subTiny(e) {
    this.subscribe(e, true);
  },
  async unsubscribe (e) {
    wx.showLoading({ title: '取消订阅中...', mask: true });
    const stream = this.getStreamByMsId(e);
    const { code } = await room.unsubscribe([stream]);
    if (code !== RCRTCCode.SUCCESS) {
        wx.showToast({ title: `unsubscribe failed: ${code}`, icon: 'none' })
    } else {
        wx.showToast({ title: 'unsubscribe success', icon: 'none' })
    }
  },
  startPreview () {
    room.startPreview()
  },
  stopPreview () {
    room.stopPreview()
  },
  async openCamera() {
    const { code } = await room.openCamera()
    if (code === RCRTCCode.SUCCESS) {
        wx.showToast({ title: `open camera success`, icon: 'none' })
    } else {
        wx.showToast({ title: `open camera failed: ${code}`, icon: 'none' })
    }
  },
  async closeCamera() {
    const { code } = await room.closeCamera()
    if (code === RCRTCCode.SUCCESS) {
        wx.showToast({ title: `close camera success`, icon: 'none' })
    } else {
        wx.showToast({ title: `close camera failed: ${code}`, icon: 'none' })
    }
  },
  async openMicrophone() {
    const { code } = await room.openMicphone()
    if (code === RCRTCCode.SUCCESS) {
        wx.showToast({ title: `open microphone success`, icon: 'none' })
    } else {
        wx.showToast({ title: `open microphone failed: ${code}`, icon: 'none' })
    }
  },
  async closeMicrophone() {
    const { code } = await room.closeMicphone()
    if (code === RCRTCCode.SUCCESS) {
        wx.showToast({ title: `close microphone success`, icon: 'none' })
    } else {
        wx.showToast({ title: `close microphone failed: ${code}`, icon: 'none' })
    }
  },
  printRoomData,
  toggleMute (e) {
    const isMute = e.detail.value
    const stream = this.getStreamByMsId(e);
    if (isMute) {
      stream.mute();
    } else {
      stream.unmute();
    }
  },
  setAudioOutputDevice (e) {
    const outputDevice = e.detail.value ? RCAudioOutputDevice.EAR : RCAudioOutputDevice.SPEAKER; 
    const stream = this.getStreamByMsId(e);
    stream.setAudioOutputDevice(outputDevice);
  },
  statechange(e) {
      console.log(`pusher state change: ${e.detail.code}`)
  },
});